/*
 ------------------------------
 Hermes FTP Server
 Copyright (c) 2006 Lars Behnke
 ------------------------------

 This file is part of Hermes FTP Server.

 Hermes FTP Server is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 Foobar is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Foobar; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.sf.hermesftp.server.impl;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;

import net.sf.hermesftp.common.FtpConstants;
import net.sf.hermesftp.common.FtpServerOptions;
import net.sf.hermesftp.exception.ConfigException;
import net.sf.hermesftp.session.impl.FtpSessionContextImpl;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

/**
 * The FTP server options used throughout the application.
 *
 * @author Lars Behnke
 *
 */
public class FtpServerOptionsImpl
    implements FtpServerOptions, FtpConstants {

    private static final int DEFAULT_BUFFER_SIZE = 1024;

    private static final int DEFAULT_FTP_IMPL_SSL_PORT = 990;

    private static final int DEFAULT_FTP_PORT = 21;

    private static final String SYS_OPT_VERSION = "version";

    private static final String SYS_OPT_TITLE = "title";

    private static final String SYS_OPT_BUILD_INFO = "info";

    private static final String APP_PROPERTIES = "/app.properties";

    private static Log log = LogFactory.getLog(FtpServerOptionsImpl.class);

    private Properties properties;

    private Properties systemProperties;

    private Set allowedPassivePorts;

    private Set occupiedPassivePorts;

    private SSLContext sslContext;

    /**
     * {@inheritDoc}
     */
    public Properties getProperties() {
        return properties;
    }

    /**
     * {@inheritDoc}
     */
    public void setProperties(Properties properties) {
        this.properties = properties;
    }

    /**
     * {@inheritDoc}
     */
    public String getProperty(String key) {
        return getProperties().getProperty(key);
    }

    /**
     * {@inheritDoc}
     */
    public int getBufferSize() {
        return getInt(OPT_BUFFER_SIZE, DEFAULT_BUFFER_SIZE);
    }

    /**
     * {@inheritDoc}
     */
    public String getRootDir() {
        File defaultDir = new File(System.getProperty("user.home"), "hermesftp");
        String dir = getString(OPT_REMOTE_DIR, defaultDir.getAbsolutePath());
        return dir;
    }

    /**
     * {@inheritDoc}
     */
    public int getFtpPort() {
        return getInt(OPT_FTP_PORT, DEFAULT_FTP_PORT);
    }

    /**
     * {@inheritDoc}
     */
    public int getImplicitSslPort() {
        return getInt(OPT_SSL_PORT_IMPLICIT, DEFAULT_FTP_IMPL_SSL_PORT);
    }

    /**
     * {@inheritDoc}
     */
    public boolean getBoolean(String optionName, boolean defaultValue) {
        boolean result;
        String boolStr = getProperties().getProperty(optionName);
        if ("true".equalsIgnoreCase(boolStr) || "false".equalsIgnoreCase(boolStr)) {
            result = Boolean.valueOf(boolStr).booleanValue();
        } else {
            result = defaultValue;
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public int getInt(String optionName, int defaultValue) {
        int result = defaultValue;
        String intStr = getProperties().getProperty(optionName);
        if (intStr != null && intStr.length() > 0) {
            try {
                result = Integer.parseInt(intStr);
            } catch (NumberFormatException e) {
                log.error("Invalid integer: " + intStr + ". Fall back to default value " + result);
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public String getString(String optionName, String defaultValue) {
        String result = getProperties().getProperty(optionName);
        if (result == null || result.length() == 0) {
            result = defaultValue;
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public String[] getStringArray(String optionName, String[] defaultValues) {
        String[] result = defaultValues;
        String strList = getProperties().getProperty(optionName);
        if (strList != null && strList.length() > 0) {
            result = strList.split(SEPARATOR);
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public int[] getIntArray(String optionName, int[] defaultValues) {
        int[] result = defaultValues;
        String strList = getProperties().getProperty(optionName);
        if (strList != null && strList.length() > 0) {
            String[] elems = strList.split(SEPARATOR);
            result = new int[elems.length];
            for (int i = 0; i < elems.length; i++) {
                result[i] = Integer.parseInt(elems[i].trim());
            }
        }
        return result;
    }

    /**
     * {@inheritDoc}
     */
    public Set getAllowedPorts() {
        if (allowedPassivePorts == null) {
            allowedPassivePorts = new HashSet();
            String[] defaultPorts = new String[] {"8000-9000"};
            String[] allowedPorts = getStringArray(OPT_ALLOWED_PASSIVE_PORTS, defaultPorts);
            for (int i = 0; i < allowedPorts.length; i++) {
                String[] portRange = allowedPorts[i].split("\\-");
                if (portRange.length > 1) {
                    int portMin = Integer.parseInt(portRange[0].trim());
                    int portMax = Integer.parseInt(portRange[1].trim());
                    for (int port = portMin; port <= portMax; port++) {
                        allowedPassivePorts.add(new Integer(port));
                    }
                } else {
                    int port = Integer.parseInt(portRange[0].trim());
                    allowedPassivePorts.add(new Integer(port));
                }
            }
        }
        return allowedPassivePorts;
    }

    /**
     * {@inheritDoc}
     */
    public int bindPassivePort() {
        synchronized (FtpServerOptionsImpl.class) {
            Set allowedPorts = getAllowedPorts();
            for (Iterator iter = allowedPorts.iterator(); iter.hasNext();) {
                Integer allowedPort = (Integer) iter.next();
                if (!getOccupiedPassivePorts().contains(allowedPort)) {
                    getOccupiedPassivePorts().add(allowedPort);
                    return allowedPort.intValue();
                }
            }
            return -1;
        }
    }

    private Set getOccupiedPassivePorts() {
        if (occupiedPassivePorts == null) {
            occupiedPassivePorts = new HashSet();
        }
        return occupiedPassivePorts;
    }

    /**
     * {@inheritDoc}
     */
    public void releasePassivePort(int port) {
        synchronized (FtpSessionContextImpl.class) {
            getOccupiedPassivePorts().remove(new Integer(port));
        }
    }

    /**
     * {@inheritDoc}
     */
    public SSLContext getSslContext() throws ConfigException {
        if (sslContext == null) {
            sslContext = createSslContext();
        }
        return sslContext;
    }

    private SSLContext createSslContext() throws ConfigException {
        SSLContext sslContext = null;
        try {
            /* Get keystore file and password */
            InputStream ksInputStream = getKeyStoreInputStream();
            char[] ksPass = getKeyStorePassword();

            /*
             * Get the java keystore object an key manager. A keystore is where keys and
             * certificates are kept.
             */
            KeyStore keystore = KeyStore.getInstance("JKS");
            keystore.load(ksInputStream, ksPass);
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("SunX509");
            kmf.init(keystore, ksPass);

            /*
             * An SSLContext is an environment for implementing JSSE. It is used to create a
             * ServerSocketFactory
             */
            sslContext = SSLContext.getInstance("SSL");
            sslContext.init(kmf.getKeyManagers(), null, null);
        } catch (KeyManagementException e) {
            throw new ConfigException("A key management authorization problem occurred.", e);
        } catch (FileNotFoundException e) {
            throw new ConfigException("The key store file could not be found.", e);
        } catch (KeyStoreException e) {
            throw new ConfigException("A key store problem occurred.", e);
        } catch (NoSuchAlgorithmException e) {
            throw new ConfigException("The hash algorithm is not supported.", e);
        } catch (CertificateException e) {
            throw new ConfigException("Certificate could not be loaded.", e);
        } catch (UnrecoverableKeyException e) {
            throw new ConfigException("Key store cannot be recovered.", e);
        } catch (IOException e) {
            throw new ConfigException("Reading the key store failed.", e);
        }
        return sslContext;
    }

    private char[] getKeyStorePassword() {
        char[] ksPass;
        String ksFile = getProperty(OPT_SSL_KEYSTORE_FILE);
        if (ksFile != null && ksFile.length() > 0) {
            String ksPassStr = getProperty(OPT_SSL_KEYSTORE_PASS);
            ksPass = ksPassStr == null ? new char[0] : ksPassStr.toCharArray();
        } else {
            ksPass = DEFAULT_KEYSTORE_PASS.toCharArray();
        }
        return ksPass;
    }

    private InputStream getKeyStoreInputStream() throws FileNotFoundException {
        String ksFile = getProperty(OPT_SSL_KEYSTORE_FILE);
        if (ksFile == null) {
            throw new FileNotFoundException("Keystore file not defined.");
        }
        return new FileInputStream(ksFile);

    }

    /**
     * {@inheritDoc}
     */
    public Properties getSystemProperties() {
        if (systemProperties == null) {
            systemProperties = new Properties();
            InputStream is = getClass().getResourceAsStream(APP_PROPERTIES);
            try {
                systemProperties.load(is);
            } catch (IOException e) {
                log.error("Could not read system properties from classpath: " + APP_PROPERTIES);
            }
        }
        return systemProperties;
    }

    /**
     * {@inheritDoc}
     */
    public String getAppTitle() {
        return getSystemProperties().getProperty(SYS_OPT_TITLE);
    }

    /**
     * {@inheritDoc}
     */
    public String getAppVersion() {
        return getSystemProperties().getProperty(SYS_OPT_VERSION);
    }

    /**
     * {@inheritDoc}
     */
    public String getAppBuildInfo() {
        return getSystemProperties().getProperty(SYS_OPT_BUILD_INFO);
    }

}
